/**
 * JHOVE2 - Next-generation architecture for format-aware characterization
 * <p>
 * Copyright (c) 2009 by The Regents of the University of California, Ithaka
 * Harbors, Inc., and The Board of Trustees of the Leland Stanford Junior
 * University. All rights reserved.
 * </p>
 * <p>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * </p>
 * <ul>
 * <li>Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.</li>
 * <li>Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.</li>
 * <li>Neither the name of the University of California/California Digital
 * Library, Ithaka Harbors/Portico, or Stanford University, nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.</li>
 * </ul>
 * <p>
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * </p>
 */

package org.jhove2.module.format.gzip;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import javax.annotation.Resource;

import org.jhove2.app.util.FeatureConfigurationUtil;
import org.jhove2.core.JHOVE2;
import org.jhove2.core.JHOVE2Exception;
import org.jhove2.core.Message;
import org.jhove2.core.io.Input;
import org.jhove2.core.source.FileSource;
import org.jhove2.core.source.Source;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;


/**
 * Tests of GZip Module
 *
 * @see org.jhove2.module.xml.XmlModule
 * @author lbihanic,selghissassi
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = {
        "classpath*:**/gzip-test-config.xml",   // GZip test beans
        "classpath*:**/jhove2-gzip-config.xml", // Standard GZip module config.
        "classpath*:**/test-config.xml",        // JHove2 test main config.
        "classpath*:**/filepaths-config.xml" }) // Test resource paths
public class GzipModuleTestBase {
    protected GzipModule sut;
    protected JHOVE2 jhove2;
    protected Source source;

    private String testDataPath;
    protected boolean initialized;

    @Resource(name = "TestGzipModule")
    public void setGzipModule(GzipModule module) {
        this.sut = module;
    }

    @Resource
    public void setJHOVE2(JHOVE2 jhove2) {
        this.jhove2 = jhove2;
    }

    @Resource
    public void setGzipDirBasePath(String path) {
        this.testDataPath = path;
    }

    /*
    protected void parse(String filePath) throws IOException, JHOVE2Exception {
        String srcPath = null;
        try {
            srcPath = FeatureConfigurationUtil.getFilePathFromClasspath(
                            this.testDataPath, "GZip sample base directory");
        }
        catch (JHOVE2Exception e) {
            fail("Could not create GZip sample base directory: " + e);
        }
        File srcDir = new File(srcPath);
        assertTrue(srcDir.exists());
        File testFile = new File(srcDir, filePath);
        assertTrue(testFile.exists());

        this.source = new FileSource(testFile);
        Input input = this.source.getInput(this.jhove2);
        this.sut.parse(this.jhove2,    this.source, input);
        this.sut.validate(this.jhove2, this.source, input);
        input.close();
    }
    */

    /*
    protected void checkChildSources(SourceDesc... descs) {
        this.checkChildSources(descs.length, new BaseValidator(descs));
    }
    */

    /*
    protected void checkChildSources(int expectedCount,
                                     SourceValidator validator) {
        List<Source> childSources = this.source.getChildSources();
        assertEquals("Parsed child sources",
                            expectedCount, childSources.size());
        assertEquals("GZip members with Deflate algorithm",
                            expectedCount, this.sut.getNumDeflateMembers());

        int i = 0;
        for (Source s : childSources) {
            assertTrue("Child source is a GzipMemberSource",
                                                s instanceof GzipMemberSource);
            if (validator != null) {
                validator.validate((GzipMemberSource)s, i++);
            }
        }
    }
    */

    /*
    protected GzipMemberSource getChildSource(int index) {
        List<Source> childSources = this.source.getChildSources();
        if (childSources.size() <= index) {
            fail("Unavailable child source: " + index
                                    + " (size=" + childSources.size() + ')');
        }
        Source s = childSources.get(index);
        assertTrue("Child source is a GzipMemberSource",
                                                s instanceof GzipMemberSource);
        return (GzipMemberSource)s;
    }
    */

    /*
    protected String getMessageCode(String name) {
        return this.sut.getClass().getName() + "." + name;
    }
    */

	/*
    protected void checkValidationMessages(Collection<Message> msgs,
                                           int expected) {
        int actual = (msgs != null)? msgs.size(): 0;
        assertEquals("Validation message count", expected, actual);
    }
    */

    /*
    protected void checkValidationMessage(String name,
                                          Collection<Message> msgs) {
        String code = this.getMessageCode(name);
        boolean found = false;
        for (Message m : msgs) {
            if (code.equals(m.getMessageCode())) {
                assertEquals("Error severity", Message.Severity.ERROR,
                                               m.getSeverity());
                assertEquals("Error context",  Message.Context.OBJECT,
                                               m.getContext());
                found = true;
            }
        }
        if (! found) {
            fail("Validation message not found: " + code);
        }
    }
    */

    /*
    protected void checkValidationMessage(String name,
                                    Collection<Message> msgs, int expected) {
        if (expected >= 0) {
            this.checkValidationMessages(msgs, expected);
        }
        if (name != null) {
            this.checkValidationMessage(name, msgs);
        }
    }
    */

    /*
    public static interface SourceValidator
    {
        public void validate(GzipMemberSource src, int index);
    }
    */

    public static class SourceDesc
    {
        public String name;
        public long offset = -1L;
        public long size  = -1L;
        public long csize = -1L;

        public Date lastModified;
        public String msg;
        public String comment;
        public int  os = -1;
        public long isize = -1L;
        public long crc16 = -1L;
        public long crc32 = -1L;

        public SourceDesc(String name, long offset, long size, long csize) {
            this(name, offset, size, csize, null, null);
        }

        public SourceDesc(String name, long offset, long size,
                                                    long csize, String date) {
            this(name, offset, size, csize, date, null);
        }

        public SourceDesc(String name, long offset, long size, long csize,
                                       String date, String errorMsg) {
            this.name = name;
            this.offset = offset;
            this.size = size;
            this.csize = csize;
            this.msg = errorMsg;

            if (date != null) {
                this.setLastModified(date);
            }
        }

        public void setLastModified(String date) {
            try {
                this.lastModified = this.getDateFormat().parse(date);
            }
            catch (Exception e) {
                throw new IllegalArgumentException(
                                    "date (" + date + "): " + e.getMessage());
            }
        }

        protected DateFormat getDateFormat() {
            DateFormat fmt = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
            fmt.setLenient(false);
            fmt.setTimeZone(TimeZone.getTimeZone("UTC"));
            return fmt;
        }
    }

    /*
    public static class BaseValidator implements SourceValidator {
        SourceDesc[] descs;

        public BaseValidator(SourceDesc... descs) {
            this.descs = descs;
        }

        public void validate(GzipMemberSource src, int index) {
            if (index >= descs.length) {
                fail("Unexpected source (" + index + "): " + src);
            }
            SourceDesc desc = descs[index];

            assertEquals("Member file name", desc.name, src.getSourceName());
            if (desc.lastModified != null) {
                assertEquals("Member last modif. date", desc.lastModified,
                                                        src.getLastModified());
            }
            assertEquals("Member comment", desc.comment, src.getComment());
            if (desc.os >= 0) {
                assertEquals("Operating system", desc.os,
                                    src.getOperatingSystem().getValue());
            }
            if (desc.offset >= 0L) {
                assertEquals("Member offset", desc.offset, src.getOffset());
            }
            if (desc.csize >= 0L) {
                assertEquals("Compressed size",
                                        desc.csize, src.getCompressedSize());
            }
            if (desc.size >= 0L) {
                assertEquals("Uncompressed size",
                                        desc.size, src.getUncompressedSize());
            }
            assertEquals("Compression ratio",
                ((desc.size > 0L) && (desc.csize > 0L))?
                    (((desc.size - desc.csize) * 100.0) / desc.size): -1.0,
                src.getCompressionRatio(), 0.01);

            if (desc.isize >= 0L) {
                assertEquals("ISIZE", desc.isize, src.getISize());
            }
            if (desc.crc16 >= 0L) {
                assertEquals("Header CRC",
                                    desc.crc16, this.hex2l(src.getCrc16()));
            }
            if (desc.crc32 >= 0L) {
                assertEquals("Data CRC",
                                    desc.crc32, this.hex2l(src.getCrc32()));
            }
            if (desc.msg != null) {
                boolean found = false;
                for (Message m : src.getMessages()) {
                    if (m.getMessageCode().equals(desc.msg)) {
                        found=true;
                        break;
                    }
                }
                if (! found) {
                    fail("Expected validation message not found: " + desc.msg);
                }
            }
        }

        protected long hex2l(String hex) {
            return ((hex != null) && (hex.length() != 0))?
                                    Long.parseLong(hex.substring(2), 16): -1L;
        }
    }
    */

}
